//
// Written by Igor Tolmachev, IT Samples
//        mailto:support@itsamples.com
//
// Copyright (c) 2009.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name and all copyright 
// notices remains intact. If the source code in this file is used in 
// any  commercial application then a statement along the lines of 
// "Portions copyright (c) Igor Tolmachev, 2009" must be included in 
// the startup banner, "About" box or printed documentation. An email 
// letting me know that you are using it would be nice as well. That's 
// not much to ask considering the amount of work that went into this.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability for any damage/loss of business that
// this product may cause.
//
// Expect bugs!
// 
// Please use and enjoy, and let me know of any bugs/mods/improvements 
// that you have found/implemented and I will fix/incorporate them into 
// this file. 

// WindowMinimizer.cpp : Defines the entry point for the application.
//

#include "stdafx.h"
#include "WindowMinimizer.h"
#include "Hook.h"
#include <stdio.h>

// Definitions
#define MAX_LOADSTRING	100
#define WM_HNOTIFYICON	(WM_USER + 1234)
#define HOTKEY			(WM_USER + 1235) 

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name
UINT nTrayMessage = RegisterWindowMessage((LPCTSTR)"GETINTRAY");

char szKey1[12] = {'b', 'g', 'h', 'j', 'k', 'l', 'q', 'u', 'w', 'x', 'y', 'z'};
char szKey2[26] = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z'};
UINT nStartUp = BST_UNCHECKED;
HWND hWnd = NULL;
BOOL bStillAbout = FALSE;
BOOL bStillSet = FALSE;

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass(HINSTANCE hInstance);
BOOL				InitInstance(HINSTANCE, int);
LRESULT CALLBACK	WndProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	Settings(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK		EnumWindowsProc(HWND hwnd, LPARAM lParam);
BOOL				TrayMessage(HWND hDlg, DWORD dwMessage, UINT uID, HICON hIcon, PSTR pszTip);

int APIENTRY _tWinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPTSTR    lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	// Initialize global strings
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);
	LoadString(hInstance, IDC_WINDOWMINIMIZER, szWindowClass, MAX_LOADSTRING);
	MyRegisterClass(hInstance);

	if(FindWindow(szWindowClass,szTitle)!=0) return FALSE;

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) return FALSE;

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_WINDOWMINIMIZER);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (int) msg.wParam;
}



ATOM MyRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_MAIN);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= NULL;
	wcex.lpszClassName	= szWindowClass;
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);

	return RegisterClassEx(&wcex);
}

BOOL TrayMessage (HWND hDlg, DWORD dwMessage, UINT uID, HICON hIcon, PSTR pszTip) 
{ 
    BOOL res = FALSE; 

    NOTIFYICONDATA tnd; 

    tnd.cbSize = sizeof(NOTIFYICONDATA); 
    tnd.hWnd = hDlg; 
    tnd.uID = uID; 

    tnd.uFlags = NIF_MESSAGE | NIF_ICON | NIF_TIP; 
    tnd.uCallbackMessage = WM_HNOTIFYICON;
    tnd.hIcon = hIcon; 

    if (pszTip) 
    { 
        lstrcpyn(tnd.szTip, pszTip, sizeof(tnd.szTip)); 
    } 
    else 
    { 
        tnd.szTip[0] = '\0'; 
    } 

    res = Shell_NotifyIcon(dwMessage, &tnd); 

    return res; 
}

BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
   hInst = hInstance; // Store instance handle in our global variable

   hWnd = CreateWindow(szWindowClass, szTitle, WS_OVERLAPPEDWINDOW,
      0, 0, 0, 0, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, SW_HIDE);
   UpdateWindow(hWnd);
   
   HICON hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(IDI_SMALL));
   TrayMessage(hWnd, NIM_ADD, 0, hIcon, "Window Minimizer v 1.2");

   StartHook();

   return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	int wmId, wmEvent;

	switch (message) 
	{
		case WM_CREATE:
         { 
			HKEY hKey = NULL;
			DWORD dwType = NULL;
			DWORD dwRes = NULL;
			DWORD dwSize = sizeof(DWORD);
			ULONG que = 0;
			SHORT nVirtualWin = 90;
			SHORT nVirtualCtrl = 65;
			LONG query = RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\ITSoftware\\WinMinimizer",0,KEY_ALL_ACCESS,&hKey);
			if (query == ERROR_SUCCESS)
			{
				que = RegQueryValueEx(hKey,"WinKey",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS && dwRes!=0)
				{
					if(dwRes>11) dwRes = 11;
					nVirtualWin = VkKeyScan(szKey1[dwRes]);
				}
				que = RegQueryValueEx(hKey,"CtrlShiftKey",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS && dwRes!=0)
				{
					if(dwRes>25) dwRes = 25;
					nVirtualCtrl = VkKeyScan(szKey2[dwRes]);
				}
				que = RegQueryValueEx(hKey,"System",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS && dwRes!=0)
				{
					RegisterHotKey(hWnd,HOTKEY,6,nVirtualCtrl); //CTRL+SHIFT
				}
				else RegisterHotKey(hWnd,HOTKEY,8,nVirtualWin);
			}
			else RegisterHotKey(hWnd,HOTKEY,8,90); // default:WIN+Z
			if(hKey) RegCloseKey(hKey);
         } 
	case WM_COMMAND:
		wmId    = LOWORD(wParam); 
		wmEvent = HIWORD(wParam); 
		// Parse the menu selections:
		switch (wmId)
		{
		case ID_APP_ABOUT:
			if(!bStillAbout) DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
			break;
		case ID_APP_EXIT:
			DestroyWindow(hWnd);
			break;
		case ID_SETTINGS:
			if(!bStillSet) DialogBox(hInst, (LPCTSTR)IDD_DIALOG_SETTINGS, hWnd, (DLGPROC)Settings);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
		}
		break;
	case WM_DESTROY:
		TrayMessage(hWnd, NIM_DELETE, 0, 0, 0); 
		UnregisterHotKey(hWnd, HOTKEY);
		StopHook();
		ShowWindow(hWnd,SW_HIDE);
		PostQuitMessage(0);
		break;
	case WM_HNOTIFYICON:
		if(lParam == WM_LBUTTONDOWN || lParam==WM_RBUTTONDOWN)
		{
			HMENU hMenu = LoadMenu(hInst,MAKEINTRESOURCE(IDM_TRAY));
			HMENU hSubMenu = GetSubMenu(hMenu,0);
			if (!hMenu) break;
			if (!hSubMenu) break;

			// Display and track the popup menu
			POINT pos;
			GetCursorPos(&pos);

			TrackPopupMenu(hSubMenu, 0, pos.x, pos.y, 0, hWnd, NULL);

			PostMessage(hWnd,WM_NULL, 0, 0);
			DestroyMenu(hMenu);
		}
		break;
    case WM_HOTKEY: 
		if(wParam==HOTKEY)
		{
			EnumWindows((WNDENUMPROC)EnumWindowsProc, 0);
		}
		break;		
 	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
	}
	return 0;
}

// Message handler for about box.
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		bStillAbout = TRUE;
		return TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) 
		{
			EndDialog(hDlg, LOWORD(wParam));
			bStillAbout = FALSE;
			return TRUE;
		}
		break;
	}
	return FALSE;
}

LRESULT CALLBACK Settings(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
	case WM_INITDIALOG:
		{
			CheckRadioButton(hDlg,IDC_RADIO_CTRLSHIFT,IDC_RADIO_WIN,IDC_RADIO_WIN);
			CheckDlgButton(hDlg,IDC_CHECK_START,BST_UNCHECKED);
			HWND hCtrlShift = GetDlgItem(hDlg,IDC_COMBO_CTRLSHIFT);
			HWND hWin = GetDlgItem(hDlg,IDC_COMBO_WIN);
			HWND hStartUp = GetDlgItem(hDlg,IDC_CHECK_START);
			LPSTR lpStr = new char[2];
			for(int i=0;i<12;i++) 
			{
				lpStr[0] = szKey1[i];
				lpStr[1] = 0;
				SendMessage(hWin,CB_ADDSTRING,0,(LPARAM)(LPCTSTR)lpStr);
			}
			for(i=0;i<26;i++) 
			{
				lpStr[0] = szKey2[i];
				lpStr[1] = 0;
				SendMessage(hCtrlShift,CB_ADDSTRING,0,(LPARAM)(LPCTSTR)lpStr);
			}
			SendMessage(hWin,CB_SETCURSEL, 11,0);
			SendMessage(hCtrlShift,CB_SETCURSEL, 0,0);

			HKEY hKey = NULL;
			DWORD dwType = NULL;
			DWORD dwRes = NULL;
			DWORD dwSize = sizeof(DWORD);
			ULONG que = 0;
			LONG query = RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\ITSoftware\\WinMinimizer",0,KEY_ALL_ACCESS,&hKey);
			if (query == ERROR_SUCCESS)
			{
				que = RegQueryValueEx(hKey,"System",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS && dwRes !=0)	CheckRadioButton(hDlg,IDC_RADIO_CTRLSHIFT,IDC_RADIO_WIN,IDC_RADIO_CTRLSHIFT);
				que = RegQueryValueEx(hKey,"StartUp",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS  && dwRes !=0) CheckDlgButton(hDlg,IDC_CHECK_START,BST_CHECKED);
				que = RegQueryValueEx(hKey,"WinKey",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS)
				{
					if(dwRes>11) dwRes = 11;
					SendMessage(hWin,CB_SETCURSEL, dwRes,0);
				}
				que = RegQueryValueEx(hKey,"CtrlShiftKey",NULL,&dwType,(LPBYTE)&dwRes,&dwSize);
				if(que==ERROR_SUCCESS)
				{
					if(dwRes>25) dwRes = 25;
					SendMessage(hCtrlShift,CB_SETCURSEL, dwRes,0);
				}

			}
			if(hKey) RegCloseKey(hKey);
			// start up options
			nStartUp = IsDlgButtonChecked(hDlg,IDC_CHECK_START);
			
			bStillSet = TRUE;
			return TRUE;
		}
	case WM_COMMAND:
			if(LOWORD(wParam) == IDOK)
			{
				HWND hCtrlShift = GetDlgItem(hDlg,IDC_COMBO_CTRLSHIFT);
				HWND hWin = GetDlgItem(hDlg,IDC_COMBO_WIN);
				HKEY hKey = NULL;
				DWORD dwType = NULL;
				ULONG value = 0;
				SHORT nVirtualWin = 90;// Y
				SHORT nVirtualCtrl = 65;// A
				UnregisterHotKey(hWnd, HOTKEY);

				LONG query = RegOpenKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\ITSoftware\\WinMinimizer",0,KEY_ALL_ACCESS,&hKey);
				if(query != ERROR_SUCCESS) query = RegCreateKeyEx(HKEY_CURRENT_USER,"SOFTWARE\\ITSoftware\\WinMinimizer",NULL,REG_NONE,REG_OPTION_NON_VOLATILE,KEY_ALL_ACCESS,NULL,&hKey,&dwType);
				if(query == ERROR_SUCCESS)
				{
					value = (LONG)SendMessage(hWin,CB_GETCURSEL,0,0);
					nVirtualWin = VkKeyScan(szKey1[value]);
					RegSetValueEx(hKey,"WinKey",NULL,REG_DWORD,(BYTE* const)&value,sizeof(DWORD));
					value = (LONG)SendMessage(hCtrlShift,CB_GETCURSEL,0,0);
					nVirtualCtrl = VkKeyScan(szKey2[value]);
					RegSetValueEx(hKey,"CtrlShiftKey",NULL,REG_DWORD,(BYTE* const)&value,sizeof(DWORD));
					value = 0;
					if(IsDlgButtonChecked(hDlg,IDC_RADIO_CTRLSHIFT) == BST_CHECKED)
					{
						RegisterHotKey(hWnd,HOTKEY,6,nVirtualCtrl);
						value = 1;
					}
					else RegisterHotKey(hWnd,HOTKEY,8,nVirtualWin);
					RegSetValueEx(hKey,"System",NULL,REG_DWORD,(BYTE* const)&value,sizeof(DWORD));
					if(IsDlgButtonChecked(hDlg,IDC_CHECK_START) == BST_CHECKED) value = 1;
					RegSetValueEx(hKey,"StartUp",NULL,REG_DWORD,(BYTE* const)&value,sizeof(DWORD));
				}
				if(hKey) RegCloseKey(hKey);

				if(nStartUp != IsDlgButtonChecked(hDlg,IDC_CHECK_START)) // modified
				{
					query = RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\Run",0,KEY_ALL_ACCESS,&hKey);
					if(query == ERROR_SUCCESS)
					{
						if(IsDlgButtonChecked(hDlg,IDC_CHECK_START)==BST_CHECKED)
						{
							char szPath[MAX_PATH]; 
							GetModuleFileName(NULL, szPath, sizeof(szPath));
							RegSetValueEx(hKey, "WinMinimizer", NULL, REG_SZ, (LPBYTE)szPath, (DWORD)strlen(szPath)); 
						}
						else RegDeleteValue(hKey, "WinMinimizer");
					}
					if(hKey) RegCloseKey(hKey);
				}
								
				EndDialog(hDlg, LOWORD(wParam));
				bStillSet = FALSE;
				return TRUE;
			}
			else if(LOWORD(wParam) == IDCANCEL)
			{
				EndDialog(hDlg, LOWORD(wParam));
				bStillSet = FALSE;
				return TRUE;
			}
			break;
	}
    return FALSE;
}

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	if(IsWindowVisible(hwnd))
	{
		TCHAR szName[MAX_LOADSTRING];
		memset(szName,0,MAX_LOADSTRING);
		GetWindowText(hwnd,(LPTSTR)szName,MAX_LOADSTRING-1);
		if(strlen(szName)>0)
		{
		
			DWORD dwStyle = GetWindowLong(hwnd,GWL_STYLE);
			DWORD dwStyleEx = GetWindowLong(hwnd,GWL_EXSTYLE);

			if(GetSystemMenu(hwnd,FALSE) 
				&& (dwStyle & WS_VISIBLE)
				&& (dwStyle & WS_MINIMIZEBOX)
				&& (dwStyle & WS_CAPTION)
				&& (dwStyle & WS_BORDER)
				&& (dwStyleEx & WS_EX_WINDOWEDGE)
				&& !(dwStyle & WS_DISABLED)
				&& !(dwStyle & WS_ICONIC)
				&& !(dwStyle & WS_CHILDWINDOW)
				&& !(dwStyleEx & WS_EX_TOOLWINDOW)
				&& !(dwStyleEx & WS_EX_MDICHILD)
				)
			{
				PostMessage(hwnd,  WM_SYSCOMMAND , nTrayMessage, 0);
				return FALSE;
			}
		}
	}

	return TRUE;
}
